﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Threading.Tasks;
using System.Web.Http;
using System.Web.Http.Description;
using Microsoft.Practices.EnterpriseLibrary.TransientFaultHandling;
using WineCloudWebApi.Data;
using WineCloudWebApi.Models;

namespace WineCloudWebApi.Controllers
{
  public class CustomerController : ApiController
  {
    // GET api/Customer
    public IList<Customer> GetCustomers()
    {
      var customers = new List<Customer>();

      Parallel.ForEach(CustomerShard.Instance.ShardRoot.Shards,
          new ParallelOptions
           { MaxDegreeOfParallelism = CustomerShard.Instance.ShardRoot.Shards.Count },
          shard =>
          {
            var shardCustomers = new List<Customer>();
            var connectionString = shard.ConnectionString;
            var retryStrategy =
             new Incremental(5, TimeSpan.FromSeconds(1), TimeSpan.FromSeconds(2));
            var retryPolicy =
             new RetryPolicy<SqlDatabaseTransientErrorDetectionStrategy>(retryStrategy);

            using (var connection = new SqlConnection(connectionString))
            {
              var commandText = "SELECT * FROM Customer";
              using (var command = new SqlCommand(commandText, connection))
              {
                connection.OpenWithRetry(retryPolicy);
                using (var reader = command.ExecuteReaderWithRetry(retryPolicy))
                {
                  while (reader.Read())
                  {
                    var customer = new Customer
                    {
                      CustomerId = Convert.ToInt32(reader["CustomerId"]),
                      FirstName = reader["FirstName"].ToString(),
                      LastName = reader["LastName"].ToString(),
                      FavoriteWineId = reader["FavoriteWineId"] as int?
                    };
                    shardCustomers.Add(customer);
                  }
                }
              }
            }
            customers.AddRange(shardCustomers);
          });

      return customers;
    }

    // GET api/Customer/5
    [ResponseType(typeof(Customer))]
    public IHttpActionResult GetCustomer(int id)
    {
      Customer customer = null;

      Shard customerShard = CustomerShard.Instance.GetShard(id);
      if (customerShard != null)
      {
        var connectionString = customerShard.ConnectionString;
        var retryStrategy =
         new Incremental(5, TimeSpan.FromSeconds(1), TimeSpan.FromSeconds(2));
        var retryPolicy =
         new RetryPolicy<SqlDatabaseTransientErrorDetectionStrategy>(retryStrategy);

        using (var connection = new SqlConnection(connectionString))
        {
          var commandText = "SELECT * FROM Customer WHERE CustomerId = @CustomerId";
          using (var command = new SqlCommand(commandText, connection))
          {
            command.Parameters.AddWithValue("@CustomerId", id);
            connection.OpenWithRetry(retryPolicy);
            using (var reader = command.ExecuteReaderWithRetry(retryPolicy))
            {
              if (reader.Read())
              {
                customer = new Customer
                {
                  CustomerId = Convert.ToInt32(reader["CustomerId"]),
                  FirstName = reader["FirstName"].ToString(),
                  LastName = reader["LastName"].ToString(),
                  FavoriteWineId = reader["FavoriteWineId"] as int?
                };
              }
            }
          }
        }
      }

      if (customer == null)
      {
        return NotFound();
      }
      return Ok(customer);
    }
  }
}
