﻿using Microsoft.Practices.EnterpriseLibrary.TransientFaultHandling;
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using System.Web.Http.Description;
using WineCloudWebApi.Models;

namespace WineCloudWebApi.Controllers
{
  public class WineController : ApiController
  {
    private WineDbContext _db = new WineDbContext();
    private RetryPolicy<SqlDatabaseTransientErrorDetectionStrategy> _retryPolicy;

    public WineController()
    {
      var retryStrategy =
       new Incremental(5, TimeSpan.FromSeconds(1), TimeSpan.FromSeconds(2));
      _retryPolicy =
       new RetryPolicy<SqlDatabaseTransientErrorDetectionStrategy>(retryStrategy);
    }

    // GET api/Wine
    public IQueryable<Wine> GetWines()
    {
      IQueryable<Wine> wines = null;
      _retryPolicy.ExecuteAction(() =>
      {
        wines = _db.Wines;
      });
      return wines;
    }

    // GET api/Wine/5
    [ResponseType(typeof(Wine))]
    public IHttpActionResult GetWine(int id)
    {
      Wine wine = null;
      _retryPolicy.ExecuteAction(() =>
      {
        wine = _db.Wines.Find(id);
      });

      if (wine == null)
      {
        return NotFound();
      }
      return Ok(wine);
    }

    // PUT api/Wine/5
    public IHttpActionResult PutWine(int id, Wine wine)
    {
      if (!ModelState.IsValid)
      {
        return BadRequest(ModelState);
      }

      if (id != wine.WineId)
      {
        return BadRequest();
      }

      _db.Entry(wine).State = EntityState.Modified;
      try
      {
        _retryPolicy.ExecuteAction(() =>
        {
          _db.SaveChanges();
        });
      }
      catch (DbUpdateConcurrencyException)
      {
        if (!WineExists(id))
        {
          return NotFound();
        }
        else
        {
          throw;
        }
      }

      return StatusCode(HttpStatusCode.NoContent);
    }

    // POST api/Wine
    [ResponseType(typeof(Wine))]
    public IHttpActionResult PostWine(Wine wine)
    {
      if (!ModelState.IsValid)
      {
        return BadRequest(ModelState);
      }

      _db.Wines.Add(wine);
      _retryPolicy.ExecuteAction(() =>
      {
        _db.SaveChanges();
      });

      return CreatedAtRoute("DefaultApi", new { id = wine.WineId }, wine);
    }

    // DELETE api/Wine/5
    [ResponseType(typeof(Wine))]
    public IHttpActionResult DeleteWine(int id)
    {
      Wine wine = null;
      _retryPolicy.ExecuteAction(() =>
      {
        wine = _db.Wines.Find(id);
      });

      if (wine == null)
      {
        return NotFound();
      }

      _db.Wines.Remove(wine);
      _retryPolicy.ExecuteAction(() =>
      {
        _db.SaveChanges();
      });

      return Ok(wine);
    }

    protected override void Dispose(bool disposing)
    {
      if (disposing)
      {
        _db.Dispose();
      }
      base.Dispose(disposing);
    }

    private bool WineExists(int id)
    {
      bool wineExists = false;
      _retryPolicy.ExecuteAction(() =>
      {
        wineExists = _db.Wines.Count(e => e.WineId == id) > 0;
      });

      return wineExists;
    }
  }
}
