﻿/*
Deployment script for WineCloudDb

This code was generated by a tool.
Changes to this file may cause incorrect behavior and will be lost if
the code is regenerated.
*/

GO
SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, CONCAT_NULL_YIELDS_NULL, QUOTED_IDENTIFIER ON;

SET NUMERIC_ROUNDABORT OFF;


GO
:setvar DatabaseName "WineCloudDb"
:setvar DefaultFilePrefix "WineCloudDb"
:setvar DefaultDataPath ""
:setvar DefaultLogPath ""

GO
:on error exit
GO
/*
Detect SQLCMD mode and disable script execution if SQLCMD mode is not supported.
To re-enable the script after enabling SQLCMD mode, execute the following:
SET NOEXEC OFF; 
*/
:setvar __IsSqlCmdEnabled "True"
GO
IF N'$(__IsSqlCmdEnabled)' NOT LIKE N'True'
    BEGIN
        PRINT N'SQLCMD mode must be enabled to successfully execute this script.';
        SET NOEXEC ON;
    END


GO
USE [$(DatabaseName)];


GO
PRINT N'Rename refactoring operation with key cd246b14-d40e-4a47-a1f0-27d7b9df3d0b is skipped, element [dbo].[Order].[Id] (SqlSimpleColumn) will not be renamed to OrderId';


GO
PRINT N'Creating [dbo].[Order]...';


GO
CREATE TABLE [dbo].[Order] (
    [OrderId]    INT           IDENTITY (1, 1) NOT NULL,
    [OrderedOn]  DATETIME2 (7) NOT NULL,
    [CustomerId] INT           NOT NULL,
    [WineId]     INT           NOT NULL,
    [Quantity]   INT           NOT NULL,
    [UnitPrice]  MONEY         NOT NULL,
    [Price]      MONEY         NOT NULL,
    [AddedOn]    DATETIME2 (7) NOT NULL,
    [UpdatedOn]  DATETIME2 (7) NULL,
    PRIMARY KEY CLUSTERED ([OrderId] ASC)
);


GO
PRINT N'Creating Default Constraint on [dbo].[Order]....';


GO
ALTER TABLE [dbo].[Order]
    ADD DEFAULT SYSDATETIME() FOR [AddedOn];


GO
PRINT N'Creating FK_Order_Customer...';


GO
ALTER TABLE [dbo].[Order] WITH NOCHECK
    ADD CONSTRAINT [FK_Order_Customer] FOREIGN KEY ([CustomerId]) REFERENCES [dbo].[Customer] ([CustomerId]);


GO
PRINT N'Creating FK_Order_Wine...';


GO
ALTER TABLE [dbo].[Order] WITH NOCHECK
    ADD CONSTRAINT [FK_Order_Wine] FOREIGN KEY ([WineId]) REFERENCES [dbo].[Wine] ([WineId]);


GO
PRINT N'Creating [dbo].[DeleteOrder]...';


GO
CREATE PROCEDURE DeleteOrder
  @OrderId int
AS

  -- Don't permit orders less than one year old to be deleted
  DECLARE @DaysOld int =
   (SELECT DATEDIFF(DAY, OrderedOn, SYSDATETIME()) FROM [Order] WHERE OrderId = @OrderId);

  -- Ensure the specified Order ID exists 
  IF @@ROWCOUNT = 0
   THROW 50000, 'The specified order was not found', 1;

  -- Ensure orders less than one year old are never deleted
  IF @DaysOld < 365
   THROW 50000, 'Orders less than one year old cannot be deleted', 1;

  -- Delete the row
  DELETE FROM [Order]
   WHERE OrderId = @OrderId;
GO
PRINT N'Creating [dbo].[InsertOrder]...';


GO
CREATE PROCEDURE InsertOrder
  @OrderedOn datetime2,
  @CustomerId int,
  @WineId int,
  @Quantity int
AS

  -- Don't permit zero or negative numbers for Quantity
  IF @Quantity < 1
   THROW 50000, 'Quantity must be 1 or more', 1;

  -- Derive unit price from Wine table 
  DECLARE @UnitPrice money = (SELECT Price FROM Wine WHERE WineId = @WineId);

  -- Ensure the specified Wine ID exists 
  IF @@ROWCOUNT = 0
   THROW 50000, 'The specified wine was not found', 1;

  -- Calculate total price
  DECLARE @Price money = @Quantity * @UnitPrice;

  -- Use the current date and time for AddedOn
  DECLARE @AddedOn datetime2 = SYSDATETIME();

  INSERT INTO [Order]
    (OrderedOn, CustomerId, WineId, Quantity, UnitPrice, Price, AddedOn)
   VALUES
    (@OrderedOn, @CustomerId, @WineId, @Quantity, @UnitPrice, @Price, @AddedOn);

  -- Return the new OrderId 
  SELECT OrderId = SCOPE_IDENTITY();
GO
PRINT N'Creating [dbo].[UpdateOrder]...';


GO
CREATE PROCEDURE UpdateOrder
  @OrderId int,
  @OrderedOn datetime2,
  @CustomerId int,
  @WineId int,
  @Quantity int
AS

  -- Don't permit zero or negative numbers for Quantity
  IF @Quantity < 1
   THROW 50000, 'Quantity must be 1 or more', 1;

  -- Derive unit price from Wine table 
  DECLARE @UnitPrice money = (SELECT Price FROM Wine WHERE WineId = @WineId);

  -- Ensure the specified Wine ID exists 
  IF @@ROWCOUNT = 0
   THROW 50000, 'The specified wine was not found', 1;

  -- Calculate total price
  DECLARE @Price money = @Quantity * @UnitPrice;

  -- Use the current date and time for UpdatedOn
  DECLARE @UpdatedOn datetime2 = SYSDATETIME();

  -- Update the row
  UPDATE [Order]
   SET
    OrderedOn = @OrderedOn,
    WineId = @WineId,
    Quantity = @Quantity,
    UnitPrice = @UnitPrice,
    Price = @Price,
    UpdatedOn = @UpdatedOn
   WHERE
    OrderId = @OrderId;
GO
