﻿using Microsoft.Phone.Controls;
using Newtonsoft.Json;
using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;

namespace WineCloudPhone
{
	public partial class MainPage : PhoneApplicationPage
	{
		private const string WebApiUrl = @"http://winecloudweb.azurewebsites.net/api/Wine";

		private Wine _wine;

		public MainPage()
		{
			InitializeComponent();
		}

		private void PhoneApplicationPage_Loaded(object sender, RoutedEventArgs e)
		{
			LoadWines();
		}

		private void LoadWines()
		{
			var wc = new WebClient();
			wc.Headers["Accept"] = "application/json";
			wc.DownloadStringCompleted += new DownloadStringCompletedEventHandler(WinesLoaded);

			// GET api/Wine
			wc.DownloadStringAsync(new Uri(WebApiUrl));
		}

		private void WinesLoaded(object sender, DownloadStringCompletedEventArgs e)
		{
			if (e.Error != null)
			{
				MessageBox.Show("Error: " + e.Error.Message);
				return;
			}

			var wines = JsonConvert.DeserializeObject<Wine[]>(e.Result);

			WineListPanel.Visibility = Visibility.Visible;
			WineDetailPanel.Visibility = Visibility.Collapsed;
			LayoutRoot.DataContext = wines;
		}

		private void NewWineButton_Click(object sender, RoutedEventArgs e)
		{
			_wine = new Wine()
			{
				Name = "New Wine",
				Year = 2004,
				Category = "New Category",
				Price = 19.99M
			};

			DeleteButton.Visibility = Visibility.Collapsed;
			WineListPanel.Visibility = Visibility.Collapsed;
			WineDetailPanel.Visibility = Visibility.Visible;
			LayoutRoot.DataContext = _wine;
		}

		private void WineListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
		{
			var addedItems = e.AddedItems as object[];
			if ((addedItems == null) || (addedItems.Length != 1))
			{
				return;
			}

			_wine = (Wine)e.AddedItems[0];

			DeleteButton.Visibility = Visibility.Visible;
			WineListPanel.Visibility = Visibility.Collapsed;
			WineDetailPanel.Visibility = Visibility.Visible;
			LayoutRoot.DataContext = _wine;
		}

		private void SaveButton_Click(object sender, RoutedEventArgs e)
		{
			var wc = new WebClient();
			wc.Headers[HttpRequestHeader.ContentType] = "application/x-www-form-urlencoded";
			wc.UploadStringCompleted += WinesUpdated;

			var wineJson = string.Format("Name={0}&Category={1}&Year={2}&Price={3}",
							HttpUtility.UrlEncode(_wine.Name),
							HttpUtility.UrlEncode(_wine.Category),
							_wine.Year,
							_wine.Price);

			if (_wine.WineId == 0)
			{
				// POST api/Wine
				wc.UploadStringAsync(new Uri(WebApiUrl), "POST", wineJson);
			}
			else
			{
				// PUT api/Wine/5
				var webApiUrl = string.Format("{0}/{1}", WebApiUrl, _wine.WineId);
				wineJson = string.Format("WineId={0}&{1}", _wine.WineId, wineJson);
				wc.UploadStringAsync(new Uri(webApiUrl), "PUT", wineJson);
			}
		}

		private void DeleteButton_Click(object sender, RoutedEventArgs e)
		{
			var confirm =
			 MessageBox.Show("Delete this wine?", "Confirm", MessageBoxButton.OKCancel);

			if (confirm != MessageBoxResult.OK)
			{
				return;
			}

			// DELETE api/Wine/5
			var wc = new WebClient();
			wc.Headers[HttpRequestHeader.ContentType] = "application/x-www-form-urlencoded";
			wc.UploadStringCompleted += WinesUpdated;

			var webApiUrl = string.Format("{0}/{1}", WebApiUrl, _wine.WineId);
			wc.UploadStringAsync(new Uri(webApiUrl), "DELETE", _wine.WineId.ToString());
		}

		private void WinesUpdated(object sender, UploadStringCompletedEventArgs e)
		{
			if (e.Error != null)
			{
				MessageBox.Show("Error: " + e.Error.Message);
				return;
			}

			LoadWines();
		}

		private void CancelButton_Click(object sender, RoutedEventArgs e)
		{
			this.LoadWines();
		}

	}
}
